package com.qmth.eds.common.contant;

import com.qmth.boot.core.uid.service.UidService;
import com.qmth.eds.common.entity.SysUser;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.util.Base64Util;
import com.qmth.eds.common.util.ServletUtil;
import org.apache.commons.io.IOUtils;

import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.util.Objects;
import java.util.StringJoiner;
import java.util.UUID;

/**
 * @Description: 系统常量
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2019/10/11
 */
public class SystemConstant {

    /**
     * 系统常量
     */
    public static final String CHARSET_NAME = "UTF-8";
    public static final Charset CHARSET = Charset.forName(CHARSET_NAME);
    public static final String MD5 = "MD5";
    public static final String SUCCESS = "success";
    public static final String USER_DIR = "user.dir";
    public static final String FILE_TEMP = "file-temp";
    public static final String SESSION = "session:";
    public static final String USER = "account";
    public static final String SCHOOL_ID = "schoolId";
    public static final String SCHOOL = "school";
    public static final String ORG = "org";
    public static final String ERROR = "/error";
    public static final String METHOD = "post";
    public static final String DEFAULT_PASSWORD = "123456";
    public static final String UPDATE_TIME = "updateTime";
    public static final String PATH = "path";
    public static final String TYPE = "type";
    public static final String LOCAL = "local";
    public static final String OSS = "oss";
    public static final String LOGO = "logo";
    public static final String ADMIN_CODE = "admin";
    public static final String UPLOAD_TYPE = "uploadType";
    public static final String HTTP = "http://";
    public static final String ID = "id";
    public static final String FILE = "file";
    public static final String SIZE = "size";
    public static final String HTML_PREFIX = ".html";
    public static final String PDF_PREFIX = ".pdf";
    public static final String FTL_PREFIX = ".ftl";
    public static final String ZIP_PREFIX = ".zip";
    public static final String JPG_PREFIX = ".jpg";
    public static final String DEFAULT_DATE_PATTERN = "yyyy-MM-dd HH:mm:ss";
    public static final int PAGE_NUMBER = 1;
    public static final int PAGE_NUMBER_MIN = 1;
    public static final int PAGE_SIZE = 100;
    public static final int PAGE_SIZE_MIN = 10;
    public static final int PAGE_SIZE_MAX = 500;
    public static final int IN_SIZE_MAX = 1000;
    public static final String DELIMITER = ":";
    public static final String COMMA = ",";
    public static final String UNDEFINED = "_";
    public static final String HYPHEN = "-";
    public static final String DEFAULT_SIGN = "#";
    public static final String AUTH = "auth";
    public static final int FINAL_SCALE = 1;
    public static final int CALCULATE_SCALE = 4;
    public static final int OPER_SCALE = 8;
    //英文字母、数字、中文括号、英文括号、下划线(_)、减号(-)、中文横线(—)
    public static final String REGULAR_EXPRESSION_OF_CODE = "[a-zA-Z0-9](\\w+)?-?(\\w+)?—?(\\w+)?(\\((\\w+)?-?(\\w+)?—?(\\w+)?\\))?(\\（(\\w+)?-?(\\w+)?—?(\\w+)?\\）)?(\\w+)?";
    public static final String REGULAR_EXPRESSION_OF_PHONE = "((\\d{3,4})|(\\(\\d{3,4}\\)-))?\\d{7,8}";

    /**
     * oss url过期时间
     */
    public static final int OSS_URL_EXPIRE = 5 * 60 * 1000;//过期时间5分钟

    /**
     * 缓存配置
     */
    public static final String USER_OAUTH_CACHE = "user:oauth:cache";
    public static final String USER_ACCOUNT_CACHE = "user:account:cache";
    public static final String USER_MENU_CACHE = "user:menu:cache";
    public static final String SCHOOL_CACHE = "school:cache";
    public static final String SCHOOL_CODE_CACHE = "school:code:cache";
    public static final String ORG_CACHE = "org:cache";
    public static final String PRIVILEGE_URL_CACHE = "privilege:url:cache";
    public static final String AUTH_INFO_CACHE = "auth:info:cache";
    public static final String ROLE_PRIVILEGE_CACHE = "role:privilege:cache";
    public static final String USER_ROLE_PRIVILEGE_CACHE = "user:role:privilege:cache";
    public static final String ROLE_CACHE = "role:cache";
    public static final String SYS_SETTING_CACHE = "sys:setting:cache";

    /**
     * 鉴权
     */
    public static final String HEADER_AUTHORIZATION = "Authorization";
    public static final String HEADER_TIME = "time";
    public static final String HEADER_PLATFORM = "platform";
    public static final String HEADER_DEVICE_ID = "deviceId";
    public static final String TOKEN = "token";

    /**
     * redis
     */
    public static final int DEFAULT_SESSION_EXPIRE = 1;//过期时间1天
    public static final long REDIS_DEFAULT_EXPIRE_TIME = 24 * 60L * 60L;//过期时间24小时

    /**
     * redis prefix
     */
    public static final String REDIS_LOCK_MQ_PREFIX = "redis:lock:mq:";

    /**
     * redis lock
     */
    public static final long REDIS_CACHE_TIME_OUT = 60L;

    /**
     * aes相关
     */
    public static final String AES = "AES";
    public static final String AES_MODE_PKCS5 = "AES/CBC/PKCS5Padding";//用这个模式，规则必须为16位
    public static final String AES_MODE_PKCS7 = "AES/CBC/PKCS7Padding";//用这个模式，规则必须为16位
    public static final String AES_RULE = "1234567890123456";//aes密钥

    /**
     * 变量
     */
    public static String TEMP_FILES_DIR;

    /**
     * excel相关
     */
    public static final String XLSX = "xlsx";
    public static final String XLS = "xls";

    /**
     * 线程池配置
     */
    public static final String THREAD_POOL_NAME = "taskThreadPool";
    //    public static final int THREAD_POOL_CORE_POOL_SIZE = 5;
//    public static final int THREAD_POOL_MAX_POOL_SIZE = 100;
    public static final int THREAD_POOL_KEEP_ALIVE_SECONDS = 10;
    public static final int THREAD_POOL_QUEUE_CAPACITY = 500;

    public static final String LOG_ERROR = "请求出错:";

    /**
     * 初始化附件文件路径
     */
    public static void initTempFiles() {
        StringJoiner localPath = new StringJoiner("").add(System.getProperty(USER_DIR));
        String mkdir = localPath.toString().substring(0, localPath.toString().lastIndexOf(File.separator));
        File fileTempDir = new File(new StringBuffer(mkdir).append(File.separator).append(FILE_TEMP).toString());
        if (!fileTempDir.exists()) {
            fileTempDir.mkdirs();
        }
        TEMP_FILES_DIR = fileTempDir.getPath();
    }

    /**
     * URL 编码, Encode默认为UTF-8.
     */
    public static String urlEncode(String part) {
        try {
            return URLEncoder.encode(part, SystemConstant.CHARSET_NAME);
        } catch (UnsupportedEncodingException e) {
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
    }

    /**
     * 图片转换
     *
     * @param imgBase64
     * @param filePath
     * @throws IOException
     */
    public static void base64ToImage(String imgBase64, String filePath) throws IOException {
        OutputStream out = null;
        try {
            byte[] b = Base64Util.decode(imgBase64);
            out = new FileOutputStream(filePath);
            IOUtils.write(b, out);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (Objects.nonNull(out)) {
                out.flush();
                out.close();
            }
        }
    }

    /**
     * id转换为long
     *
     * @return
     */
    public static Long convertIdToLong(String id) {
        return Objects.nonNull(id) && id.length() > 0 ? Long.parseLong(id) : null;
    }

    /**
     * id转换为Integer
     *
     * @return
     */
    public static Integer convertIdToInteger(String id) {
        return Objects.nonNull(id) && id.length() > 0 ? Integer.parseInt(id) : null;
    }

    /**
     * 获取全局uuid
     *
     * @return
     */
    public static String getUuid() {
        return String.valueOf(UUID.randomUUID()).replaceAll("-", "");
    }

    /**
     * 获取数据库uuid
     *
     * @return
     */
    public static Long getDbUuid() {
        UidService uidservice = SpringContextHolder.getBean(UidService.class);
        return uidservice.getId();
    }

    /**
     * enable转换
     *
     * @return
     */
    public static Boolean convertEnable(Boolean enable) {
        return Objects.nonNull(enable) ? enable : true;
    }

    public static Long getHeadOrUserSchoolId() {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long schoolId;
        if (Objects.nonNull(ServletUtil.getRequestHeaderSchoolIdByNotVaild())) {
            schoolId = Long.parseLong(ServletUtil.getRequestHeaderSchoolIdByNotVaild().toString());
        } else {
            schoolId = sysUser.getSchoolId();
        }
        return schoolId;
    }
}