package com.qmth.eds.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.entity.ExamScheduleTask;
import com.qmth.eds.common.entity.ExamType;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.util.ServletUtil;
import com.qmth.eds.mapper.ExamTypeMapper;
import com.qmth.eds.service.ExamScheduleTaskService;
import com.qmth.eds.service.ExamTypeService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Service
public class ExamTypeServiceImpl extends ServiceImpl<ExamTypeMapper, ExamType> implements ExamTypeService {

    @Resource
    private ExamScheduleTaskService examScheduleTaskService;

    @Override
    public boolean saveData(ExamType examType) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        if (examType.getId() == null) {
            examType.setId(SystemConstant.getDbUuid());
            examType.setSchoolId(schoolId);
            return this.save(examType);
        } else {
            QueryWrapper<ExamType> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ExamType::getSchoolId, schoolId)
                    .eq(ExamType::getName, examType.getName())
                    .ne(ExamType::getId, examType.getId());
            List<ExamType> examTypes = this.list(queryWrapper);
            if (!examTypes.isEmpty()) {
                throw ExceptionResultEnum.ERROR.exception("名称已存在");
            }
            return this.updateById(examType);
        }
    }

    @Override
    public boolean delete(Long id) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<ExamScheduleTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamScheduleTask::getSchoolId, schoolId)
                .eq(ExamScheduleTask::getExamTypeId, id);
        List<ExamScheduleTask> examScheduleTasks = examScheduleTaskService.list(queryWrapper);
        if (!examScheduleTasks.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("考试类型已在定时任务中使用，不能删除");
        }
        return this.removeById(id);
    }

    @Override
    public IPage<ExamType> pageData(Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Page<ExamType> page = new Page<>(pageNumber, pageSize);
        QueryWrapper<ExamType> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamType::getSchoolId, schoolId);
        return this.page(page, queryWrapper);
    }

    @Override
    public List<ExamType> listData() {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<ExamType> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamType::getSchoolId, schoolId)
                .eq(ExamType::getEnable, true);
        return this.list(queryWrapper);
    }

    @Override
    public boolean enable(Long id) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        ExamType examType = this.getById(id);

        if (examType.getEnable()) {
            long currentTime = System.currentTimeMillis();
            QueryWrapper<ExamScheduleTask> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ExamScheduleTask::getSchoolId, schoolId)
                    .eq(ExamScheduleTask::getExamTypeId, id)
                    .lt(ExamScheduleTask::getStartTime, currentTime)
                    .gt(ExamScheduleTask::getEndTime, currentTime);
            int count = examScheduleTaskService.count(queryWrapper);
            if (count != 0) {
                throw ExceptionResultEnum.ERROR.exception("该考试类型下有未结束的定时任务，无法禁用");
            }
        }

        examType.setEnable(!examType.getEnable());
        return this.updateById(examType);
    }
}
